;;;; (net http client) -- client library for HTTP.

;;  Copyright 2001, Evan Prodromou <evan@prodromou.san-francisco.ca.us>.

;;  This program is free software; you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation; either version 2 of the License, or
;;  (at your option) any later version.

;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.

;;  You should have received a copy of the GNU General Public License
;;  along with this program; if not, write to the Free Software
;;  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

(define-module (net http util)
  :use-module (ice-9 regex))

(export parse-uri urlencode http-date copy-port copy-port-count)

;; Compatibility kludge.

(or (defined? 'read-line)
    (use-modules (ice-9 rdelim)))

;;; Takes a URI, returns (protocol server port rest-of-stuff)
;;; XXX: Break this out into a separate library, eh?
;;; XXX: Support full URL syntax

(define (parse-uri uri)
  (let ((uri-match (string-match "(.*)://([^:/]*)(:([0-9]+))?(/.*)" uri)))
    (if (not uri-match)
	#f
	(let* ((protocol (match:substring uri-match 1))
	       (server (match:substring uri-match 2))
	       (portstr (match:substring uri-match 4))
	       (port (if portstr (string->number portstr) 80))
	       (rest (match:substring uri-match 5)))
	  (list protocol server port rest)))))

;; XXX: This really needs to be separated into another module.

(define (urlencode str)
  (let ((outstr ""))
    (do ((i 0 (+ 1 i)))
	((= i (string-length str)) outstr)
      (let ((c (string-ref str i)))
	(cond
	 ((or (char-alphabetic? c) (char-numeric? c))
          (set! outstr (string-append outstr (string c))))
	 ((equal? c #\space)
          (set! outstr (string-append outstr (string #\+))))
	 (t
	  (set! outstr (string-append outstr (string-upcase (format #f "%~2,'0X" (char->integer c)))))))))))

(define (http-date gmt)
  (strftime "%a, %2d %b %C%g %T %Z" gmt))

(define (copy-port in out)
  (let copy-char ((c (read-char in)))
    (if (eof-object? c)
	'()
	(begin
	  (write-char c out)
	  (copy-char (read-char in))))))

(define (copy-port-count in out cnt)
  (do ((total 0 (+ 1 total)))
      ((>= total cnt) '())
    (let ((c (read-char in)))
      (if (eof-object? c)
	  '()
	  (write-char c out)))))

